/*
** Module   : filesys.c
** Abstract : implementation for detection of simbolic links, non-FIFO pipes
**            and special devices.
**            This is used by com.goldencode.p2j.utilFileChecker class
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------Description----------------------
** 001 CS  20130104 Created initial version for Implement OS-DIR support as
**                  an INPUT stream task.
** 002 CS  20130115 Task 1613: finish FILE-INFO system handle support (added
**                  native function for getting file creation date time).
** 003 EVL 20130329 Making changes to compile Windows DLL.
*/ 
#include "filesys.h"

#include "com_goldencode_p2j_util_FileChecker.h"

/**
* Utility method to check the type of the given file.
* There are 4 main types a file can be: Directory , File , Device and Unknown.
* In addition to the 4 main types there are 3 additive stats (they can be 
* added to the main type. For example: FH -> File hidden, DL -> Directory link.
* In some cases the additive status can only be added to a specific main type.
* Ex: the Progress behaviour for hidden attribute -> only hidden file are 
* marked as hidden, hidden directories are only marked as directory.
*
* @param  env 
*         pointer to a structure that provides access to JNI environment
* @param  jc 
*         reference to the Java class who called this function
*         (from static context)
* @param  fname 
*         The file name to be checked passed from FileChecker.checkFileType()
*
* @return A wrapped integer representing the status code of the file.
*/
JNIEXPORT jint JNICALL 
       Java_com_goldencode_p2j_util_FileChecker_checkFileType(JNIEnv  *env,
                                                              jclass  jc, 
                                                              jstring fname) 
{
   const char *file_path = (*env)->GetStringUTFChars(env, fname, NULL);
   if (file_path == NULL) 
   {
      return -1; /* OutOfMemoryError already thrown */
   }
   
   int status = UNIX;
   struct stat st;
   int fd = 0;
   if ((fd = open_native(file_path, O_RDONLY)) < 0)
   {
     return -1;
   }
   
   // Use fstat to detect files and directories or links that point to files
   // and directories as files or directories
   if (fstat(fd, &st)) 
   { 
      // Close the file 
      close(fd);
      return -1; 
   }
   
   // check if regular file
   if (S_ISREG(st.st_mode)) 
   {
      // File status is standalone status
      status |= FILE;
   }
   // extract the filename from the path
   // replicate the Unix 4GL bug in which upper dir appears as hidden
   // by nontreating .. special case when checking for hidden.
   char *filename = (strrchr(file_path, '/'))+1;
   // in unix hidden files don't have a property that determines if
   // they are hidden instead a hidden file will start with '.'
   //check if first letter is '.'
   if (*filename == '.')
   {
      // hidden status is additive status
      status |= HIDDEN;
   }
   
   // check if file is directory
   if (S_ISDIR(st.st_mode)) 
   { 
      status |= DIR;
   } 
   // check for fifo pipe
   if (S_ISFIFO(st.st_mode))
   {
      // Pipe code status is additive but may not detected as file
      // File code status is also added if
      if (!(status & FILE))
      {
         status |= FILE;
      }
      status |= PIPE;
   }
   
   // Close the file 
   close(fd);
   // Get the pratform specific sym link status
   status = getSymLinkStatus(env, file_path, status);
   // Free Java memory
   (*env)->ReleaseStringUTFChars(env, fname, file_path);
   
   return status;  
}

/**
* Utility method that will return the creation date of a file 
* represented as milliseconds from year 1970.
* 
* @param  env 
*         pointer to a structure that provides access to JNI environment
* @param  jc 
*         reference to the Java class who called this function
*         (from static context)
* @param  fname 
*         The file name to be checked passed from 
*         FileChecker.getFileCreationTime.
* 
* @return A wrapped integer representing the  file creation time in 
*         milliseconds from 1970 or -1 if operation has failed , 
*         (real data cannot be -1 since precision
*         for returned date is at the level of seconds).
*/
JNIEXPORT jlong JNICALL 
   Java_com_goldencode_p2j_util_FileChecker_getFileCreationTime(JNIEnv  *env,
                                                                jclass  jc, 
                                                                jstring fname) 
{
   const char *file_path = (*env)->GetStringUTFChars(env, fname, NULL);
   if (file_path == NULL) 
   {
      return -1; /* OutOfMemoryError already thrown */
   }
   struct stat st; 
   if (stat(file_path, &st)) 
   {
      return -1; 
   }
   // the return variable must be long long otherwise it will truncate
   // on Unix the only date that is kept is last modify time
   long long m = (long long) st.st_ctime;
   return m * 1000L;
}
